<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use SupNext\Util\Util;
use SupNext\Services\QuoteSheetGrid\ImportCollectionOrderService as Service;

use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Environment;

use Doctrine\ORM\EntityManager;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\XLS\Table;

use PHPExcel_Cell;
use PHPExcel_Worksheet;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;

class ImportCollectionOrderController {
    private $util;
    private $service;
    private $errors = [];
    private $newCode;
    private $entityManager;
    private $environment;

    public function __construct(Util $util, Service $service, NewCode $newCode, $entityManager, Environment $environment) {
        $this->util = $util;
        $this->service = $service;
        $this->newCode = $newCode;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }

    public function importCollectionOrder(Request $request, Response $response) {
        try {
            $data = $request->getParameters();

            $tableRows = $this->readXls($data);
            $rows = $this->collectionOrder($tableRows, $data);

            $rows = array_splice($rows, 1);

            if ($rows != null) {
                return $response->setError(new Error($rows, 400));
            }

            return $response->addDataSet(
                new DataSet("collectionOrder",
                [
                    "message"     => "Importação planilha de cotacao realizada com sucesso",
                    "status_code" => 200
                ]));
        } catch (\Exception $exception) {
            throw new \Exception("Erro ao importar pedido de coleta: " . $exception->getMessage(), 500);
        }
    }

    private function readXls(array $data) {
        try {
            $file = $data["FILE"];
            $xlsFile = FileLoader::loadFromBase64($file["content"], $file["name"]);

            if (!$xlsFile) throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);

            $worksheet = $xlsFile->getActiveSheet();
            $table = $this->formatXls($worksheet);

            return $table;
        } catch (\Exception $exception) {
            throw new \Exception("Erro ao ler xlsx: " . $exception->getMessage(), 500);
        }
    }

    private function formatXls($worksheet) {
        try {
            $minRowIndex = 2;
            $worksheetDimensions = $worksheet->getHighestRowAndColumn();
            $maxRowIndex = $worksheetDimensions["row"];
            $columnNames = [
                "REGIAO",
                "NOME_REGIAO",
                "VALIDADE_INI",
                "VALIDADE_FIN",
                "COMPRADOR",
                "NOME_COMPRADOR",
                "FORNECEDOR",
                "NM_FORNECEDOR",
                "PRODUTO",
                "NOME_PR",
                "MARCA",
                "NOME_MARCA",
                "QTDE",
                "VRUNI",
                "OBS_FORNECEDOR"
            ];

            $maxColumnIndex = $this->getColumnIndex($worksheetDimensions["column"], $worksheet, $columnNames);
            $columnsReference = $this->buildColumnNameReference($worksheet, $maxColumnIndex, $columnNames);
            $tableRows = [];

            if (!count($columnsReference)) throw new \Exception("sup::FILE_WITH_ERRORS");

            array_pop($columnNames);

            for ($row = $minRowIndex; $row <= $maxRowIndex; $row++) {
                $worksheetRowContent = [];
                $rowIsEmpty = true;

                for ($column = 0; $column <= $maxColumnIndex; $column++) {
                    $columnLetterIndex = PHP_VERSION_ID >= 70400
                        ? Coordinate::stringFromColumnIndex($column)
                        : PHPExcel_Cell::stringFromColumnIndex($column);
                    $columnName = $columnsReference[$columnLetterIndex];
                    $cellValue = $worksheet->getCell($columnLetterIndex . $row)->getValue();

                    if (!is_null($cellValue)) {
                        $rowIsEmpty = false;
                        $cellValue = $worksheet->getCell($columnLetterIndex . $row)->getFormattedValue();

                        switch ($columnLetterIndex) {
                            case "A":
                                $cellValue = str_pad($cellValue, 3, '0', STR_PAD_LEFT);
                                break;
                            case "E":
                                $cellValue = str_pad($cellValue, 12, '0', STR_PAD_LEFT);
                                break;
                            case "G":
                            case "I":
                                $cellValue = substr($cellValue, 1);
                                break;
                            case "K":
                                $cellValue = str_pad($cellValue, 5, '0', STR_PAD_LEFT);
                                break;
                            case "M":
                            case "N":
                                $cellValue = (float)$cellValue;
                                break;
                            default:
                                break;
                        }
                    }

                    if (is_null($cellValue) && in_array($columnName, $columnNames)) {
                        $this->setError($row, $columnName, $cellValue, "sup::NOT_INFORMED");
                    }

                    $worksheetRowContent = array_merge($worksheetRowContent, [$columnName => $cellValue]);
                }

                if ($this->errors) {
                    count($this->errors) != 1 ? array_shift($this->errors) : null;
                    $worksheetRowContent["ERROS"] = $this->errors[0];
                }

                if (!$rowIsEmpty) {
                    $worksheetRow = [
                        'index'   => $row,
                        'content' => $worksheetRowContent
                    ];

                    array_push($tableRows, $worksheetRow);
                }
            }


            $resultTable = new Table($columnsReference, $tableRows);
            $rowsFromTable = $resultTable->getRows();

            return $rowsFromTable;
        } catch (\Exception $exception) {
            throw new \Exception("Erro ao formatar xlsx: " . $exception->getMessage(), 500);
        }
    }

    private function setError($rowNum, $columnName, $value, $message, $secondValue = null) {
        array_push(
            $this->errors,
            [
                "ROWNUM"       => $rowNum,
                "COLUMN"       => $columnName,
                "VALUE"        => $value,
                "ERROR_CODE"   => $message,
                "SECOND_VALUE" => $secondValue
            ]
        );
    }

    private function getColumnIndex($column, $worksheet, $columnNames) {
        if (gettype($column) == 'string') {
            if (PHP_VERSION_ID >= 70400) {
                $table = FileReader::readWorksheetTable($worksheet, $columnNames);
                return $table->getNumberOfColumns();
            }

            return PHPExcel_Cell::columnIndexFromString($column) - 1;
        }

        return $column;
    }

    private function buildColumnNameReference($worksheet, $maxColumn, $columnNames) {
        $reference = [];

        if (self::isHeaderSizeRight($maxColumn, $columnNames)) {
            for ($column = 0; $column <= $maxColumn; $column++) {
                $columnString = self::getColumnString($column);
                $columnName = $worksheet->getCell($columnString . '1')->getValue();
                $reference[$columnString] = mb_strtoupper($columnName, 'UTF-8');
            }
        }

        return $reference;
    }

    private function isHeaderSizeRight($maxColumn, $columnNames) {
        return PHP_VERSION_ID >= 70400
            ? ($maxColumn == count($columnNames))
            : ($maxColumn == count($columnNames) - 1);
    }

    private static function getColumnString($column) {
        if (gettype($column) == 'integer') {
            return PHP_VERSION_ID >= 70400
                ? Coordinate::stringFromColumnIndex($column)
                : PHPExcel_Cell::stringFromColumnIndex($column);
        }

        return $column;
    }

    private function collectionOrder($rows, $data) {
        try {
            $this->entityManager->beginTransaction();

            $cdfilial = $this->util->getCentralCompras($data["CDFILIAL"]);
            $nrColeta = $this->newCode->getCodeByTableName("COLETA" . $cdfilial, $this->environment->getNrOrgTrab());
            $primeiraVez = true;
            $cdforneced = $this->getCDFORNECED($rows[0]["content"]["FORNECEDOR"], $data["NRORG"]);

            foreach ($rows as $row) {
                $content = $row["content"];
                $erros = $content["ERROS"] ?? null;

                if (!isset($row[0])) {
                    if ($erros) {
                        $marca = $this->verificaMarca($cdfilial, $data["NRORG"]);
                        $produto = $this->verificaProduto($content["PRODUTO"], $data["NRORG"]);
                        $valor = $content["VRUNI"];

                        if ($valor >= 0 && $valor != null) {
                            if ($marca) {
                                $regiao = $this->verificaRegiao($content["REGIAO"], $cdforneced, $data["NRORG"]);

                                if (!$regiao) {
                                    $cdproduto = $this->adquireProdutoFornecedor($cdforneced, $produto, $content["VALIDADE_FIN"], $data["NRORG"], $cdfilial);

                                    if ($cdproduto) {
                                        $cdprmarcompl = $this->adquireMarcaFornecedor($marca, $cdproduto, $content["VALIDADE_FIN"], $data["NRORG"], $cdfilial);

                                        if ($cdprmarcompl) {
                                            $itemcole = $this->verificaItemColeta($cdprmarcompl, $cdforneced, $content["VALIDADE_FIN"], $data["NRORG"], $cdfilial);

                                            if (!$itemcole) {
                                                if ($primeiraVez) {
                                                    $itemcole = $this->acmInsertColeta($content, $cdfilial, $nrColeta, $marca["IDUTISTATCOT"]);
                                                    $primeiraVez = false;
                                                }
                                                $cdsAuxiliar = $this->acmAuxiliar($cdforneced, $data["NRORG"]);
                                                $this->acmInsertItemCole($content, $produto, $cdfilial, $nrColeta, $marca["IDUTISTATCOT"], $cdforneced, $cdsAuxiliar);
                                            } else {
                                                $cdsAuxiliar = $this->acmAuxiliar($cdforneced, $data["NRORG"]);
                                                $this->acmUpdateItemCole($itemcole, $content, $cdsAuxiliar, $cdforneced);
                                            }

                                            $this->procImposto($content, $cdfilial, $nrColeta, $cdforneced, $produto);
                                            $this->acmForcTrigger($cdfilial, $nrColeta, $content, $cdforneced, $produto);
                                        } else {
                                            $this->setError($row["index"], $content["PRODUTO"], null, "sup::PRODUCT_NOT_REGISTERED_FOR_BUYER_BRAND_PARAMS", $content["MARCA"]);
                                        }
                                    } else {
                                        $this->setError($row["index"], $content["PRODUTO"], null, "sup::PRODUCT_NOT_REGISTERED_FOR_BUYER_SUPPLIER_PARAMS", $content["FORNECEDOR"]);
                                    }
                                } else {
                                    $this->setError($row["index"], $content["FORNECEDOR"], null, "sup::SUPPLIERS_NOT_SERVING_REGIONS_PARAMS", $content["REGIAO"]);
                                }
                            } else {
                                $this->setError($row["index"], $content["MARCA"], null, "sup::BRAND_NOT_REGISTERED");
                            }
                        } else {
                            $this->setError($row["index"], null, $content["PRODUTO"], "sup::PROD_UNIT_PRICE_INVALID");
                            if ($erros != null) {
                                $this->setError($row["index"], $erros["COLUMN"], null, "sup::NOT_INFORMED");
                            }
                        }
                    }
                }
            }

            $this->entityManager->commit();

            return $this->errors ?? null;
        } catch (Exception $e) {
            $this->entityManager->rollback();
            throw new \Exception('sup::ERROR_IMPORTING_COLLECTION');
        }
    }

    private function getCDFORNECED($nrinsjurforn, $nrorg) {
        return $this->util->filteredDataSet([
            "NRINSJURFORN" => $nrinsjurforn,
            "NRORG"        => $nrorg
        ], "getCDFORNECED")[0]["CDFORNECED"];
    }

    private function verificaMarca($cdfilial, $nrorg) {
        return $this->util->filteredDataSet([
            "CDFILIAL" => $cdfilial,
            "NRORG"    => $nrorg
        ], "verifyBrand")[0]["CDMARCA"];
    }

    private function verificaProduto($produto, $nrorg) {
        return $this->util->filteredDataSet([
            "CDARVPROD" => $produto,
            "NRORG"     => $nrorg
        ], "verifyProduct")[0]["CDPRODUTO"];
    }

    private function verificaRegiao($regiao, $cdforneced, $nrorg) {
        return $this->util->filteredDataSet([
            "CDREGIAO"   => $regiao,
            "CDFORNECED" => $cdforneced,
            "NRORG"      => $nrorg
        ], "verifyRegion");
    }

    private function adquireProdutoFornecedor($cdforneced, $produto, $validadeFin, $nrorg, $cdfilial) {
        return $this->util->filteredDataSet([
            "CDFORNECED"  => $cdforneced,
            "CDPRODUTO"   => $produto,
            "NRORG"       => $nrorg,
            "CDFILIAL"    => $cdfilial,
            "DTFINVALITM" => $validadeFin,
        ], "adqProdFornComp")[0]["CDPRODUTO"];
    }

    private function adquireMarcaFornecedor($marca, $cdproduto, $validadeFin, $nrorg, $cdfilial) {
        return $this->util->filteredDataSet([
            "CDMARCA"     => $marca,
            "CDFILIAL"    => $cdfilial,
            "CDPRODUTO"   => $cdproduto,
            "NRORG"       => $nrorg,
            "DTFINVALITM" => $validadeFin
        ], "adqQfoProdForn");
    }

    private function verificaItemColeta($cdprmarcompl, $cdforneced, $validadeFin, $nrorg, $cdfilial) {
        return $this->util->filteredDataSet([
            "CDFILIAL"    => $cdfilial,
            "CDMARCA"     => $cdprmarcompl,
            "CDFORNECED"  => $cdforneced,
            "CDPRODUTO"   => $cdprmarcompl,
            "DTFINVALITM" => $validadeFin,
            "NRORG"       => $nrorg
        ], "adqVerifItemCole");
    }
}
