<?php

namespace SupNext\Util;

use Doctrine\DBAL\Connection;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class Util
{
    const TABLE_LISTA = "LISTA";

    public function __construct(Connection $connection, CrudService $crudService, Environment $environment, NewCode $newCode)
    {
        $this->connection  = $connection;
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->newCode     = $newCode;
    }

    public function getSessionData(): array
    {
        $sessionData = array(
            "NRORG" => $this->environment->getNrorg(),
            "CDOPERADOR" => $this->environment->getCdOperador(),
            "CDFILIAL" => $this->environment->getCdFilial(),
            "CDFILCENTCMP" => $this->getCentralCompras()
        );

        return $sessionData;
    }

    public function getDateProc()
    {
        $params = array(
            'P_CDFILIAL' => $this->environment->getCdFilial()
        );

        $query = "SELECT CASE WHEN NVL(G.IDUTILDTPROC,'N') = 'S'
                              THEN TRUNC(NVL(P.DTPROCESSA,SYSDATE))
                              ELSE TRUNC(SYSDATE)
                         END DTPROCESSA
                    FROM PARAMFILIAL P, PARAMGERAL G
                   WHERE P.CDFILIAL = :P_CDFILIAL
                     AND G.CDPARGERAL = '0001'
                     AND P.NRORG = G.NRORG";

        $std = $this->connection->prepare($query);
        $std->execute($params);
        $result = $std->fetchAll()[0];
        $dataProc = substr($result["DTPROCESSA"], 0, 10);

        return $dataProc;
    }

    public function prepareConditions(array $params): array
    {
        $sessionData = $this->getSessionData();
        $sessionData['DTPROCESSA'] = $dataProc = $this->getDateProc();
        $conditions = [];

        foreach ($params as $param) {
            array_push($conditions, array(
                "columnName" => $param,
                "operator"   => '=',
                "value"      => $sessionData[$param]
            ));
        }
        return $conditions;
    }


    /**
     * Start transaction on database server
     *
     * @access public
     * @author Jefferson Gouvea
     */
    public function beginTransaction(): void
    {
        $this->connection->beginTransaction();
    }

    /**
     * Commit transaction on database server
     *
     * @access public
     * @author Jefferson Gouvea
     */
    public function commit(): void
    {
        $this->connection->commit();
    }

    /**
     * Roll back transaction in database
     *
     * @access public
     * @author Jefferson Gouvea
     */
    public function rollback(): void
    {
        $this->connection->rollback();
    }

    public function insertLista($cdlistaArray)
    {
        if (gettype($cdlistaArray) == 'string') {
            $cdlistaArray = [$cdlistaArray];
        }

        $dataSourceName = strtolower(self::TABLE_LISTA);
        $nrlista = $this->newCode->getCodeByTableName(self::TABLE_LISTA, $this->environment->getNrOrg());
        $sql = "INSERT INTO " . self::TABLE_LISTA . " (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";

        foreach ($cdlistaArray as $cdlista) {
            $params = array(
                "NRLISTA" => $nrlista,
                "CDLISTA" => $cdlista
            );
            $query = $this->connection->prepare($sql);
            $query->execute($params);
        }

        return $nrlista;
    }

    public function deleteLista(string $nrlista): void
    {
        $conditions = array(
            "NRLISTA" => $nrlista
        );

        $delete = "DELETE FROM " . self::TABLE_LISTA . " WHERE NRLISTA = :NRLISTA";
        $std = $this->connection->prepare($delete);
        $std->execute($conditions);
    }

    public function getCentralCompras()
    {
        $data["CDFILIAL"] = $this->environment->getCdFilial();

        $query = "SELECT F2.CDFILIAL AS CDFILCENTCMP FROM FILIAL F, FILIAL F2 WHERE F.CDFILIAL = :P_CDFILIAL AND F2.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $stmt = $this->connection->prepare($query);

        $stmt->execute(["P_CDFILIAL" => $data["CDFILIAL"]]);

        return $stmt->fetch()["CDFILCENTCMP"];
    }

    /**
     * Execute a newCode on a table name received.
     *
     * @param string $tableName
     */

    public function generateCode($tableName)
    {
        $newCode = $this->newCode->getCodeByTableName($tableName, $this->environment->getNrOrgTrab());

        return $newCode;
    }

    /**
     * Receive a query and param and check if operation is a "SELECT",
     * if is, return the result. The param passed along with query are
     * not mandatory.
     *
     * @param $query
     * @param $params
     * @return array
     * @throws \Exception
     */

    public function crud(string $query, array $params = [])
    {
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);

            if (self::getQueryOperation($query)) return $std->fetchAll(\PDO::FETCH_ASSOC);
        } catch (\Exception $exception) {
            throw $exception;
        }
    }

    /**
     * Returns true if query operation is a "SELECT" and false if
     * is another one.
     *
     * @param string $query
     */

    public function getQueryOperation($query)
    {
        return strpos(strtoupper(ltrim($query)), 'SELECT') === 0 ? true : false;
    }

    public function addCaracterInRight(string $string): string
    {
        $string = str_pad($string, 20, "z", STR_PAD_RIGHT);

        return $string;
    }

    public function implodeFields(array $data, array $fields): array
    {
        foreach ($fields as $field) {
            $data[$field] = empty($data[$field]) ? null : implode("_", $data[$field]);
        }

        return $data;
    }


    public function createConditions(array $data): array
    {
        $conditions = [];

        foreach ($data as $columnName => $value) {
            $conditions[] = [
                "columnName" => $columnName,
                "operator"   => "=",
                "value"      => $value
            ];
        }

        return $conditions;
    }

    public function filteredDataSet(array $conditionsArray, string $datasourceName): array
    {
        $conditions = $this->createConditions($conditionsArray);

        $data = $this->crudService
            ->getFilteredDataSet($datasourceName, $conditions)
            ->getRows();

        return $data;
    }

    public function createListValues($lista)
    {
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_', $lista);

        try {
            foreach ($valores as $cdlista) {
                $this->insertList($nrlista, $cdlista);
            }

            return $nrlista;
        } catch (\Exception $exception) {
            throw new \Exception("Erro ao criar lista de valores: " . $exception->getMessage(), 500);
        }
    }

    private function insertList($nrlista, $cdlista): void
    {
        $query = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";

        $params = [
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        ];

        $this->crud($query, $params);
    }
}
