/**
 * Classe Singleton que mantém as variáveis de estado da aplicação.
 * Esta classe garante que exista apenas uma instância em toda a aplicação.
 *
 * Responsável por:
 * - Armazenar variáveis de aplicação, como IDSISDOS, registeredParams, CDFILCENTCMP, CDOPERADOR, CDFILIAL e NRORG.
 * - Fornecer um método para inicializar essas variáveis obtendo dados através da classe Utilities.
 * - Garantir que as variáveis sejam inicializadas apenas uma vez e estejam acessíveis de qualquer parte da aplicação.
 *
 * Uso:
 * - Chame AppState.getInstance() para obter a única instância desta classe.
 * - Use o método initialize para configurar as variáveis de estado com dados obtidos do Utilities.
 * - Acesse as variáveis de estado diretamente da instância após a inicialização.
 */

import { Utilities } from '@/util/Utilities';

export class AppState {
  private static instance: AppState;

  public readonly registeredParams: any;

  public readonly CDFILCENTCMP: string;

  public readonly CDOPERADOR: string;

  public readonly CDFILIAL: string;

  public readonly IDSISDOS: string;

  public readonly NRORG: string;

  private constructor() {
    // Construtor privado para impedir a instanciação direta
  }

  public static getInstance(): AppState {
    if (!AppState.instance) {
      AppState.instance = new AppState();
    }

    return AppState.instance;
  }

  public async initialize(utilities: Utilities): Promise<void> {
    try {
      const data = await utilities.getLocalVar();

      (this as any).registeredParams = data.REGISTERED_PARAMS;
      (this as any).CDFILCENTCMP = data.CDFILCENTCMP;
      (this as any).CDOPERADOR = data.CDOPERADOR;
      (this as any).CDFILIAL = data.CDFILIAL;
      (this as any).IDSISDOS = data.IDSISDOS;
      (this as any).NRORG = data.NRORG;
    } catch (error) {
      utilities.showDialog('Erro ao inicializar variáveis locais.', 'error');
    }
  }
}
