import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  Date as DateInput,
  ModalService,
  Form,
  LoadingService,
  Select,
  SelectMultiple,
  Checkbox,
  Number as NumberInput,
  Button,
  Grid,
  ITabsBeforeChangeEventParam,
  Tabs,
  TextInput,
} from '@zeedhi/common';
import {
  Metadata,
  IEventParam,
  Http,
  I18n,
} from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';

export class SpreadsheetInformationController {
  private appState: AppState;

  private utilities!: Utilities;

  private fieldsValues: any;

  private historyTrading: any;

  constructor() {
    this.appState = AppState.getInstance();
    this.utilities = new Utilities();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/spreadsheetInformationModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('spreadsheetInformationModal')?.destroy();
  }

  public openModal(): void {
    const productGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsProductsGrid',
    );

    if (!productGrid.selectedRows.length || productGrid.selectedRows.length > 1) {
      this.utilities.showDialog(
        'Selecione um produto para acessar informações da planilha.',
        'warning',
      );

      return;
    }

    ModalService.getModal('spreadsheetInformationModal').show();

    setTimeout(() => {
      const tab = Metadata.getInstance<Tabs>('spreadsheetInformationTab');

      if (tab.activeTab !== 0) {
        while (tab.activeTab !== 0) {
          tab.previousTab();
        }
      }

      this.getFieldsValues();
    }, 100);
  }

  public closeModal(): void {
    ModalService.getModal('spreadsheetInformationModal').hide();
  }

  private async getFieldsValues(): Promise<void> {
    const quoteGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsQuotationGrid',
    );
    const regionGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsRegionGrid',
    );
    const supplierGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsSupplierGrid',
    );
    const productGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsProductsGrid',
    );

    const parameters = {
      CDFILIAL:
        quoteGrid.datasource.currentRow.CDFILIAL ?? this.appState.CDFILIAL,
      CDFORNECED: supplierGrid.datasource.currentRow.CDFORNECED,
      CDMARCA: productGrid.selectedRows[0].CDMARCA,
      CDPRODUTO: productGrid.selectedRows[0].CDPRODUTO,
      CDREGIAO: regionGrid.datasource.currentRow.CDREGIAO,
      NRCOLEPRECO: quoteGrid.datasource.currentRow.NRCOLEPRECO,
      VRPRECUNIITM: productGrid.selectedRows[0].VRPRECUNIITM,
      NRORG: this.appState.NRORG,
    };

    LoadingService.show();

    try {
      const { data } = await Http.get('/getFieldsAdditionalData', {
        params: parameters,
      });
      this.fieldsValues = data.dataset.getFieldsValue;

      this.setFieldsAdditionalData();
      LoadingService.hide();
    } catch (error) {
      LoadingService.hide();
      console.error(error);
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  private setFieldsAdditionalData(): void {
    const form = Metadata.getInstance<Form>('additionalDataForm');

    Object.assign(form.value, {
      chargeType: this.fieldsValues[0].NMTIPOCOBR,
      discount: this.fieldsValues[0].VRPERDESITM,
      IDTIPFREITM: this.fieldsValues[0].IDTIPFREITM,
      shippingCost: this.fieldsValues[0].VRFRETEFORN,
      RDPROD: this.fieldsValues[1].RDPROD,
      RDMARCA: this.fieldsValues[1].RDMARCA,
      RDREGIAO: this.fieldsValues[1].RDREGIAO,
      RDFORNECED: this.fieldsValues[1].RDFORNECED,
      productSpreadsheet: [this.fieldsValues[0].CDARVPROD],
      brandSpreadsheet: [this.fieldsValues[0].CDMARCA],
      regionSpreadsheet: [this.fieldsValues[0].CDREGIAO],
      supplierSpreadsheet: [this.fieldsValues[0].CDFORNECED],
    });
  }

  public async setFieldsTabs({
    component,
  }: ITabsBeforeChangeEventParam): Promise<void> {
    setTimeout(async () => {
      switch (component.activeTab) {
        case 1:
          this.setFieldsPaymentMethodForm();
          break;
        case 3:
          this.setFieldsTradingForm();
          break;
        case 4:
          await this.setFieldsHistory();
          this.setHistoryFieldsVisibility();
          break;
        default:
          break;
      }
    }, 200);
  }

  private setFieldsPaymentMethodForm() {
    const form = Metadata.getInstance<Form>('paymentMethodForm');

    switch (this.fieldsValues[0].IDTPFOPGTCOL) {
      case 'V':
        this.enableGrid();
        this.paymentMethod();
        break;
      case 'F':
        this.disableGrid();
        this.fixPayment();
        break;
      default:
        this.disableGrid();
        this.disableFields();
        break;
    }

    Object.assign(form.value, {
      paymentOptions: this.fieldsValues[0].IDTPFOPGTCOL ?? 'S',
      dayPaymentMethod: this.fieldsValues[0].NRDIAPGFXCOL,
      installmentsPaymentMethod: this.fieldsValues[0].NRPARCPGTCOL,
      firstInstallment: this.fieldsValues[0].IDUTDIAPGCOL,
      CDFORMPGTO: this.fieldsValues[0].CDFORMPGTO,
    });
  }

  private enableGrid(): void {
    const grid = Metadata.getInstance<Grid>('paymentMod');
    const saveButton = Metadata.getInstance<Button>('savePaymentMethod');

    grid.isVisible = true;
    saveButton.disabled = false;
  }

  private paymentMethod(): void {
    const saveButton = Metadata.getInstance<Button>('savePaymentMethod');
    const fields = [
      'dayPaymentMethod',
      'installmentsPaymentMethod',
      'firstInstallment',
      'CDFORMPGTO',
    ].map(
      (field) => Metadata.getInstance<Select>(field)
        || Metadata.getInstance<NumberInput>(field),
    );

    fields.forEach((field) => {
      if (field) {
        field.isVisible = field.name === 'CDFORMPGTO';
      }
    });

    saveButton.disabled = false;
  }

  private fixPayment(): void {
    const saveButton = Metadata.getInstance<Button>('savePaymentMethod');
    const fields = [
      'dayPaymentMethod',
      'installmentsPaymentMethod',
      'firstInstallment',
      'CDFORMPGTO',
    ].map(
      (field) => Metadata.getInstance<Select>(field)
        || Metadata.getInstance<NumberInput>(field),
    );

    fields.forEach((field) => {
      if (field) {
        field.isVisible = !(field.name === 'CDFORMPGTO');
      }
    });

    saveButton.disabled = false;
  }

  private disableGrid(): void {
    const grid = Metadata.getInstance<Grid>('paymentMod');
    const cancelButton = Metadata.getInstance<Button>('cancelPaymentMethod');
    const saveButton = Metadata.getInstance<Button>('savePaymentMethod');

    grid.isVisible = false;
    cancelButton.isVisible = true;
    saveButton.disabled = true;
  }

  private disableFields(): void {
    const fields = [
      'paymentOptions',
      'dayPaymentMethod',
      'installmentsPaymentMethod',
      'firstInstallment',
      'CDFORMPGTO',
    ].map(
      (field) => Metadata.getInstance<Select>(field)
        || Metadata.getInstance<NumberInput>(field),
    );

    fields.forEach((field) => {
      if (field) {
        field.disabled = true;
        field.isVisible = field.name === 'paymentOptions';
      }
    });
  }

  private async setFieldsTradingForm(): Promise<void> {
    const tradingForm = Metadata.getInstance<Form>('trading_form');

    if (!this.historyTrading) {
      await this.fetchHistoryTrading();
    }

    Object.assign(tradingForm.value, {
      new_unit_price: this.historyTrading.ITEMCOLE.VRPRECUNIITM,
      trading_discount: this.historyTrading.ITEMCOLE.VRPERDESITMAT ?? 0,
      trading_discount2:
        this.historyTrading.ITEMCOLE.VRPERDESITMAT > 0
        && this.historyTrading.ITEMCOLE.VRPERDESITMAT > 0
          ? (this.historyTrading.ITEMCOLE.VRPRECUNIITM
              * this.historyTrading.ITEMCOLE.VRPERDESITMAT)
            / 100
          : 0,
      economic: this.historyTrading.ITEMCOLE.VRPRECECOITM,
      current_unit_price: this.historyTrading.ITEMCOLE.VRPRECUNIITM || 0,
      current_unit_price2: null,
      current_trading_discount: this.historyTrading.ITEMCOLE.VRPERDESITM || 0,
      current_trading_discount2:
        this.historyTrading.ITEMCOLE.VRPRECUNIITMAT > 0
        && this.historyTrading.ITEMCOLE.VRPERDESITMAT > 0
          ? (this.historyTrading.ITEMCOLE.VRPRECUNIITMAT
              * this.historyTrading.ITEMCOLE.VRPERDESITMAT)
            / 100
          : 0,
      current_economic: this.historyTrading.ITEMCOLE.VRPRECECOITMAT || 0,
      current_economic2: null,
      trading_RDPROD: this.fieldsValues[1].RDPROD,
      trading_product: [this.fieldsValues[0].CDARVPROD],
      trading_RDMARCA: this.fieldsValues[1].RDMARCA,
      trading_brand: [this.fieldsValues[0].CDMARCA],
      trading_RDREGIAO: this.fieldsValues[1].RDREGIAO,
      trading_region: [this.fieldsValues[0].CDREGIAO],
      trading_RDFORNECED: this.fieldsValues[1].RDFORNECED,
      trading_supplier: [this.fieldsValues[0].CDFORNECED],
    });
  }

  private setHistoryFieldsVisibility(): void {
    const LastQuotationPaymentMethod = Metadata.getInstance<TextInput>(
      'last_quotation_payment_method',
    );
    const LastQuotationNMTPFORPGTCLUCF = Metadata.getInstance<NumberInput>(
      'last_quotation_NMTPFORPGTCLUCF',
    );
    const LastQuotationNRDIAPGFIXFOUCF = Metadata.getInstance<NumberInput>(
      'last_quotation_NRDIAPGFIXFOUCF',
    );
    const LastQuotatoinPlot = Metadata.getInstance<NumberInput>('last_quotation_plot');

    LastQuotationNMTPFORPGTCLUCF.isVisible = LastQuotationPaymentMethod.value === 'V'
      || LastQuotationPaymentMethod.value === 'F';
    LastQuotationNRDIAPGFIXFOUCF.isVisible = LastQuotationPaymentMethod.value === 'F';
    LastQuotatoinPlot.isVisible = LastQuotationPaymentMethod.value === 'F';

    const fields = [
      {
        paymentMethod: Metadata.getInstance<TextInput>(
          'last_purchase_payment_method',
        ),
        paymentMethod2: Metadata.getInstance<TextInput>(
          'last_purchase_payment_method2',
        ),
        NMTPFORPGTCLUCF: Metadata.getInstance<NumberInput>(
          'last_purchase_NRDIAPGFIXFOUCF',
        ),
        plot: Metadata.getInstance<NumberInput>('last_purchase_plot'),
      },
      {
        paymentMethod: Metadata.getInstance<TextInput>(
          'lowest_price_payment_method',
        ),
        paymentMethod2: Metadata.getInstance<TextInput>(
          'lowest_price_payment_method2',
        ),
        NMTPFORPGTCLUCF: Metadata.getInstance<NumberInput>(
          'lowest_price_NRDIAPGFIXFOUCF',
        ),
        plot: Metadata.getInstance<NumberInput>('lowest_price_plot'),
      },
      {
        paymentMethod: Metadata.getInstance<TextInput>(
          'lowest_price_alt_payment_method',
        ),
        paymentMethod2: Metadata.getInstance<TextInput>(
          'lowest_price_alt_payment_method2',
        ),
        NMTPFORPGTCLUCF: Metadata.getInstance<NumberInput>(
          'lowest_price_alt_NRDIAPGFIXFOUCF',
        ),
        plot: Metadata.getInstance<NumberInput>('lowest_price_alt_plot'),
      },
    ];

    fields.forEach((field) => {
      const {
        paymentMethod,
        paymentMethod2,
        NMTPFORPGTCLUCF,
        plot,
      } = field;

      NMTPFORPGTCLUCF.isVisible = paymentMethod.value === 'V' || paymentMethod.value === 'F';
      paymentMethod2.isVisible = paymentMethod.value === 'V' || paymentMethod.value === 'F';
      plot.isVisible = paymentMethod.value === 'F';

      paymentMethod.value = this.setFieldPaymentMethod(paymentMethod.value);
    });
  }

  private setFieldPaymentMethod(paymentMethod: string): string {
    const paymentMethodMapping = {
      V: I18n.translate('Pela forma de pagamento'),
      F: I18n.translate('Pagamento fixo no dia'),
      null: I18n.translate('Sem forma de pagamento'),
    };

    return paymentMethodMapping[paymentMethod] || '';
  }

  private async setFieldsHistory(): Promise<void> {
    const form = Metadata.getInstance<Form>('history_form');

    if (!this.historyTrading) {
      await this.fetchHistoryTrading();
    }

    this.populateFormWithHistoryData(form);
  }

  private async fetchHistoryTrading(): Promise<void> {
    const productGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsProductsGrid',
    );
    const additionalDataForm = Metadata.getInstance<Form>('additionalDataForm');

    const parameters = {
      NRCOLEPRECO: productGrid.selectedRows[0].NRCOLEPRECO,
      CDFILIAL: this.appState.CDFILIAL,
      CDFORNECED: productGrid.selectedRows[0].CDFORNECED,
      CDMARCA: productGrid.selectedRows[0].CDMARCA,
      CDPRODUTO: productGrid.selectedRows[0].CDPRODUTO,
      CDREGIAO: productGrid.selectedRows[0].CDREGIAO,
      DTFINVALITM: additionalDataForm.value.additionalDataValidity[0],
      DTINIVALITM: additionalDataForm.value.additionalDataValidity[1],
      NRORG: this.appState.NRORG,
    };

    LoadingService.show();

    try {
      const { data } = await Http.get('/getHistoryTrading', {
        params: parameters,
      });
      this.historyTrading = data.dataset.historyTrading;
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', 'error');
    } finally {
      LoadingService.hide();
    }
  }

  private populateFormWithHistoryData(form: Form): void {
    const historyDataMapping = [
      { field: 'last_quotation', dataKey: 'ULTCOTFORN' },
      { field: 'last_purchase', dataKey: 'ULTCMP' },
      { field: 'lowest_price', dataKey: 'MINPRECNOR' },
      { field: 'lowest_price_alt', dataKey: 'MINPRECALT' },
      { field: 'original_price', dataKey: 'ITEMCOLE' },
    ];

    historyDataMapping.forEach(({ field, dataKey }) => {
      const data = this.historyTrading[dataKey] || {};

      form.value[`${field}_unit_price`] = data.VRPRECUNIITM ?? 0;
      form.value[`${field}_economic_price`] = data.VRPRECUNIITM ?? 0;
      form.value[`${field}_NMTPFORPGTCLUCF`] = data.NMFORMPGTO ?? 0;
      form.value[`${field}_NRDIAPGFIXFOUCF`] = data.NRDIAPGFXCOL ?? 0;
      form.value[`${field}_plot`] = data.NRPARCPGTCOL ?? 0;
      form.value[`${field}_payment_method`] = data.IDTPFOPGTCOL ?? '';
    });

    form.value.original_price_unit_price = this.historyTrading.ITEMCOLE.VRPRECUNORIG ?? 0;
    form.value.original_price_economic_price = this.historyTrading.ITEMCOLE.VRPRECECORIG ?? 0;
    form.value.original_price_econimic_price_pi = this.historyTrading.ITEMCOLE.VRPRECPIORIG ?? 0;
  }

  public setValidity({ component }: IEventParam<DateInput>): void {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    component.value = [
      grid.datasource.currentRow.DTINIVALCOL.substring(0, 10),
      grid.datasource.currentRow.DTFINVALCOL.substring(0, 10),
    ];
  }

  public get filterChargeType() {
    return {
      P_CDTIPCOB: 'T',
      P_NRORG: this.appState.NRORG,
    };
  }

  public get filterFields() {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    return {
      P_CDFILIAL: grid.datasource.currentRow.CDFILIAL,
      P_NRCOLEPRECO: grid.datasource.currentRow.NRCOLEPRECO,
    };
  }

  public changeFieldsStateAdditionalData({ component }: IEventParam<Checkbox>) {
    const fields = ['RDPROD', 'RDMARCA', 'RDREGIAO', 'RDFORNECED'].map(
      (field) => Metadata.getInstance<Select>(field),
    );

    const multipleFields = [
      'productSpreadsheet',
      'brandSpreadsheet',
      'regionSpreadsheet',
      'supplierSpreadsheet',
    ].map((field) => Metadata.getInstance<SelectMultiple>(field));

    const isEnabled = component.value === 'S';

    fields.forEach((field) => {
      field.disabled = !isEnabled;
    });

    if (!isEnabled) {
      multipleFields.forEach((field) => {
        field.disabled = true;
      });
    }

    if (isEnabled) {
      fields.forEach((field) => {
        this.validateAdditionalDataFieldDiscount({ component: field });
      });
    }
  }

  public validateAdditionalDataFieldDiscount({
    component,
  }: IEventParam<Select>): void {
    const replicateDiscount = Metadata.getInstance<Checkbox>('IDCARGOBS');

    if (replicateDiscount.value !== 'S') return;

    const fieldsMap = {
      RDPROD: 'productSpreadsheet',
      RDMARCA: 'brandSpreadsheet',
      RDREGIAO: 'regionSpreadsheet',
      RDFORNECED: 'supplierSpreadsheet',
    };

    const field = Metadata.getInstance<SelectMultiple>(
      fieldsMap[component.name],
    );
    field.disabled = component.value !== 'L';
  }

  public setDiscount({ component }: IEventParam<NumberInput>) {
    const productGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsProductsGrid',
    );
    const form = Metadata.getInstance<Form>('additionalDataForm');
    const unitPrice = parseFloat(productGrid.selectedRows[0].VRPRECUNIITM) || 0;
    const discount = parseFloat(component.value) || 0;
    let result = 0;

    if (component.name !== 'discount') {
      result = unitPrice !== 0 ? (discount * 100) / unitPrice : 0;
    } else {
      result = (unitPrice * discount) / 100;
    }

    form.value.VRPRECDESITM = result;
  }

  public async saveAdditionalData(): Promise<void> {
    const parameters = this.prepareParams();

    try {
      await Http.put('/saveAdditionalData', { params: parameters });

      this.closeModal();
      this.utilities.showDialog('sup::SUCCESS', 'success');
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  private prepareParams() {
    const form = Metadata.getInstance<Form>('additionalDataForm');
    const productGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsProductsGrid',
    );

    return {
      CDFILIAL: productGrid.selectedRows[0].CDFILIAL,
      CDFILCENT: this.appState.CDFILCENTCMP,
      NRCOLEPRECO: productGrid.selectedRows[0].NRCOLEPRECO,
      CDMARCA: productGrid.selectedRows[0].CDMARCA,
      CDPRODUTO: productGrid.selectedRows[0].CDPRODUTO,
      CDARVPROD: productGrid.selectedRows[0].CDARVPROD,
      CDFORNECED: productGrid.selectedRows[0].CDFORNECED,
      CDREGIAO: productGrid.selectedRows[0].CDREGIAO,
      CDTIPCOB: form.value.chargeType,
      TCDTIPCOB: form.value.chargeType ? null : 'T',
      DTCOLEPREC: null,
      CDSEGMARCCOL: form.value.secondBrand ?? null,
      VRFRETEFORN: form.value.shippingCost ?? 0,
      VRPRECUNIITM: productGrid.selectedRows[0].VRPRECUNIITM,
      IDTIPFREITM: form.value.IDTIPFREITM,
      CDMARCASEC: form.value.secondBrand,
      REPLICADESCONTO: form.value.IDCARGOBS,
      RDMARCA: form.value.RDMARCA,
      CDMARCAPES:
        form.value.brandSpreadsheet.length > 0
          ? form.value.brandSpreadsheet.join('_')
          : 'T',
      RDREGIAO: form.value.RDREGIAO,
      CDREGIAOPES:
        form.value.regionSpreadsheet.length > 0
          ? form.value.regionSpreadsheet.join('_')
          : 'T',
      RDFORNECED: form.value.RDFORNECED,
      CDFORNECEDPES:
        form.value.supplierSpreadsheet.length > 0
          ? form.value.supplierSpreadsheet.join('_')
          : 'T',
      RDPROD: form.value.RDPROD,
      CDARVPRODPES:
        form.value.productSpreadsheet.length > 0
          ? form.value.productSpreadsheet.join('_')
          : 'T',
      VRPERDESITM: form.value.discount,
      IDDESCPROD:
        Array.isArray(form.value.productSpreadsheet)
        && form.value.productSpreadsheet.length > 1
          ? 2
          : 1,
      IDDESCMARCA:
        Array.isArray(form.value.brandSpreadsheet)
        && form.value.brandSpreadsheet.length > 1
          ? 2
          : 1,
      IDDESCREG:
        Array.isArray(form.value.regionSpreadsheet)
        && form.value.regionSpreadsheet.length > 1
          ? 2
          : 1,
      IDDESCFORN:
        Array.isArray(form.value.supplierSpreadsheet)
        && form.value.supplierSpreadsheet.length > 1
          ? 2
          : 1,
      ...this.fieldsValues[0],
      DTFINVALITM: this.fieldsValues[0].DTFINVALITM.substring(0, 10),
      DTINIVALITM: this.fieldsValues[0].DTINIVALITM.substring(0, 10),
      NRORG: this.appState.NRORG,
    };
  }

  public get setFilterPaymentMod() {
    return {
      P_CDFILIAL: this.fieldsValues[0].CDFILIAL,
      P_NRCOLEPRECO: this.fieldsValues[0].NRCOLEPRECO,
      P_CDFORNECED: this.fieldsValues[0].CDFORNECED,
      P_CDPRODUTO: this.fieldsValues[0].CDPRODUTO,
      P_CDMARCA: this.fieldsValues[0].CDMARCA,
      P_CDREGIAO: this.fieldsValues[0].CDREGIAO,
      P_NRORG: this.appState.NRORG,
    };
  }

  public async updatePaymentMethod(): Promise<void> {
    const parameters = this.prepareParamsUpdateMethod();

    try {
      await Http.put('/updatePaymentMethod', { params: parameters });

      this.closeModal();
      this.utilities.showDialog('sup::SUCCESS', 'success');
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  private prepareParamsUpdateMethod() {
    const productGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsProductsGrid',
    );
    const quoteGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsQuotationGrid',
    );
    const additionalDataForm = Metadata.getInstance<Form>('additionalDataForm');
    const paymentMethodForm = Metadata.getInstance<Form>('paymentMethodForm');
    const { currentRow } = productGrid.datasource;
    const paymentModGrid = Metadata.getInstance<Grid>('paymentMod');

    return [
      { ...paymentModGrid.datasource.data },
      {
        CDFILIAL: currentRow.CDFILIAL,
        CDFORMPGTO: paymentMethodForm.value.CDFORMPGTO,
        CDFORNECED: currentRow.CDFORNECED,
        CDMARCA: currentRow.CDMARCA,
        CDPRODUTO: currentRow.CDPRODUTO,
        CDREGIAO: currentRow.CDREGIAO,
        CDSEGMARCCOL: this.fieldsValues[0].CDSEGMARCCOL,
        CDTIPCOB: additionalDataForm.value.chargeType,
        DTINIVALITM: quoteGrid.datasource.currentRow.DTINIVALCOL.substring(
          0,
          10,
        ),
        DTFINVALITM: quoteGrid.datasource.currentRow.DTFINVALCOL.substring(
          0,
          10,
        ),
        IDTIPFREITM: additionalDataForm.value.IDTIPFREITM,
        IDTPFOPGTCOL: paymentMethodForm.value.paymentOptions,
        IDUTDIAPGCOL: paymentMethodForm.value.firstInstallment,
        NRCOLEPRECO: currentRow.NRCOLEPRECO,
        NRDIAPGFXCOL: paymentMethodForm.value.dayPaymentMethod,
        NRPARCPGTCOL: paymentMethodForm.value.installmentsPaymentMethod,
        NRPZOENTCOL: this.fieldsValues[0].NRPZOENTCOL,
        QTACMPITMCOL: this.fieldsValues[0].QTACMPITMCOL,
        VRPRECECOITM: currentRow.VRPRECECOITM,
        VRPRECUNIITM: currentRow.VRPRECUNIITM,
        NRORG: this.appState.NRORG,
      },
    ];
  }

  public changeFieldsStateTrading({ component }: IEventParam<Checkbox>) {
    const fields = [
      'trading_RDPROD',
      'trading_RDMARCA',
      'trading_RDREGIAO',
      'trading_RDFORNECED',
    ].map((field) => Metadata.getInstance<Select>(field));

    const multipleFields = [
      'trading_product',
      'trading_brand',
      'trading_region',
      'trading_supplier',
    ].map((field) => Metadata.getInstance<SelectMultiple>(field));

    const isEnabled = component.value === 'S';

    fields.forEach((field) => {
      field.disabled = !isEnabled;
    });

    if (!isEnabled) {
      multipleFields.forEach((field) => {
        field.disabled = true;
      });
    }

    if (isEnabled) {
      fields.forEach((field) => {
        this.validateTradingFieldDiscount({ component: field });
      });
    }
  }

  public validateTradingFieldDiscount({
    component,
  }: IEventParam<Select>): void {
    const replicateDiscount = Metadata.getInstance<Checkbox>('trading_IDCARGOBS');

    if (replicateDiscount.value !== 'S') return;

    const fieldsMap = {
      trading_RDPROD: 'trading_product',
      trading_RDMARCA: 'trading_brand',
      trading_RDREGIAO: 'trading_region',
      trading_RDFORNECED: 'trading_supplier',
    };

    const field = Metadata.getInstance<SelectMultiple>(
      fieldsMap[component.name],
    );
    field.disabled = component.value !== 'L';
  }

  public setDiscountTrading({ component }: IEventParam<NumberInput>) {
    const form = Metadata.getInstance<Form>('trading_form');
    const unitPrice = parseFloat(form.value.new_unit_price) || 0;
    const discount = parseFloat(form.value.trading_discount) || 0;
    const discount2 = parseFloat(form.value.trading_discount2) || 0;

    if (
      component.name !== 'trading_discount'
      && component.name !== 'new_unit_price'
    ) {
      form.value.trading_discount = unitPrice !== 0 ? (discount2 * 100) / unitPrice : 0;
    } else {
      form.value.trading_discount2 = (unitPrice * discount) / 100;
    }
  }

  public async simulateTrading(): Promise<void> {
    const tradingForm = Metadata.getInstance<Form>('trading_form');

    if (!tradingForm.validate()) return;

    if (
      tradingForm.value.new_unit_price
      === this.historyTrading.ITEMCOLE.VRPRECUNIITM
    ) {
      this.utilities.showDialog(
        'Um novo preço unitário deve ser informado.',
        'warning',
      );

      return;
    }

    const productGrid = Metadata.getInstance<TekGrid>(
      'multipleLevelsProductsGrid',
    );
    const additionalDataForm = Metadata.getInstance<Form>('additionalDataForm');
    const parameters = {
      CDFILIAL: productGrid.selectedRows[0].CDFILIAL,
      CDFILCENT: this.appState.CDFILCENTCMP,
      NRCOLEPRECO: productGrid.selectedRows[0].NRCOLEPRECO,
      CDMARCA: productGrid.selectedRows[0].CDMARCA,
      CDPRODUTO: productGrid.selectedRows[0].CDPRODUTO,
      CDARVPROD: productGrid.selectedRows[0].CDARVPROD,
      CDFORNECED: productGrid.selectedRows[0].CDFORNECED,
      CDREGIAO: productGrid.selectedRows[0].CDREGIAO,
      REPLICADESCONTO: tradingForm.value.trading_IDCARGOBS,
      RDMARCA: tradingForm.value.trading_RDMARCA,
      CDMARCAPES:
        tradingForm.value.trading_brand.length > 0
          ? tradingForm.value.trading_brand.join('_')
          : 'T',
      RDREGIAO: tradingForm.value.trading_RDREGIAO,
      CDREGIAOPES:
        tradingForm.value.trading_region.length > 0
          ? tradingForm.value.trading_region.join('_')
          : 'T',
      RDFORNECED: tradingForm.value.trading_RDFORNECED,
      CDFORNECEDPES:
        tradingForm.value.trading_supplier.length > 0
          ? tradingForm.value.trading_supplier.join('_')
          : 'T',
      RDPROD: tradingForm.value.trading_RDPROD,
      CDARVPRODPES:
        tradingForm.value.trading_product.length > 0
          ? tradingForm.value.trading_product.join('_')
          : 'T',
      IDDESCPROD:
        Array.isArray(additionalDataForm.value.productSpreadsheet)
        && additionalDataForm.value.productSpreadsheet.length > 1
          ? 2
          : 1,
      IDDESCMARCA:
        Array.isArray(additionalDataForm.value.brandSpreadsheet)
        && additionalDataForm.value.brandSpreadsheet.length > 1
          ? 2
          : 1,
      IDDESCREG:
        Array.isArray(additionalDataForm.value.regionSpreadsheet)
        && additionalDataForm.value.regionSpreadsheet.length > 1
          ? 2
          : 1,
      IDDESCFORN:
        Array.isArray(additionalDataForm.value.supplierSpreadsheet)
        && additionalDataForm.value.supplierSpreadsheet.length > 1
          ? 2
          : 1,
      VRPERDESITM: tradingForm.value.trading_discount,
      VRPRECECOITM: tradingForm.value.trading_economic,
      VRPRECUNIITM: tradingForm.value.new_unit_price,
      NRORG: this.appState.NRORG,
    };

    LoadingService.show();

    try {
      await Http.put('/calculateTrading', { params: parameters });

      this.utilities.showDialog('sup::SUCCESS', 'success');
      this.setFieldsTradingForm();
      LoadingService.hide();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  public enableTrading(): void {
    const fields = ['new_unit_price', 'trading_discount', 'trading_discount2'];
    const { tabs } = Metadata.getInstance<Tabs>('spreadsheetInformationTab');
    const simulateButton = Metadata.getInstance<Button>('simulate_trading');

    fields.forEach((field) => {
      Metadata.getInstance<NumberInput>(field).disabled = false;
    });
    tabs.forEach((tab) => {
      tab.disabled = false;
    });

    simulateButton.label = 'Simular';
  }
}
