import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  DialogService,
  LoadingService,
} from '@zeedhi/common';
import {
  Metadata,
  Http,
  I18n,
  IEventParam,
  Datasource,
} from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';
import { AppStateController } from '@/controllers/quoteSheetGrid/AppStateController';
import { ModalsController } from '@/controllers/quoteSheetGrid/ModalsController';

export class QuoteSheetGridController {
  private utilities!: Utilities;

  private appState: AppState;

  private modals: ModalsController;

  private departmentCurrentRow: any = {};

  private employeeCurrentRow: any = {};

  constructor() {
    this.utilities = new Utilities();
    this.appState = new AppStateController().getAppState();
    this.modals = new ModalsController();
  }

  public onChangeDepartmentCurrentRow({ component }: IEventParam<Datasource>) {
    this.departmentCurrentRow = component.currentRow;
  }

  public onChangeEmployeeCurrentRow({ component }: IEventParam<Datasource>) {
    this.employeeCurrentRow = component.currentRow;
  }

  public async createModals(): Promise<void> {
    this.modals.createModals();
  }

  public destroyModals(): void {
    this.modals.destroyModals();
  }

  public async updateQuotation({ component, event }: any): Promise<void> {
    event.preventDefault();

    const row = component.getEditedRows();
    row.IDDISMONTLIS = row.IDDISMONTLIS === 'Sim' ? 'S' : 'N';

    this.utilities.updateRow(component, row, component.datasource.route);
  }

  public async updatePaymentMethod(): Promise<void> {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!grid.selectedRows.length) {
      this.utilities
        .showDialog(
          'Selecione uma cotação para realizar a atualização da forma de pagamento.',
          'warning',
        );

      return;
    }

    try {
      await Http.post('/savePaymentMethod', grid.selectedRows);
      this.utilities.showDialog('sup::SUCCESS', 'success');
      grid.reload();
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  public async deleteRows({ component, event }: any): Promise<void> {
    event.preventDefault();

    if (component.name === 'multipleLevelsSupplierGrid') {
      const region = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');

      component.selectedRows.forEach((row) => {
        row.CDREGIAO = region.datasource.currentRow.CDREGIAO;
      });
    }

    this.utilities.deleteSelectedRows(component, component.selectedRows, component.datasource.route);
  }

  public async deleteRegion({ component, event }: any): Promise<void> {
    event.preventDefault();
    LoadingService.show();

    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const parameters = component.selectedRows.map((row) => ({
      ...row,
      CDFILIAL: quoteGrid.datasource.currentRow.CDFILIAL,
    }));

    try {
      await Http.delete('/deleteRegion', { params: parameters });

      LoadingService.hide();

      this.utilities.showDialog('sup::SUCCESS', 'success');

      component.selectedRows = [];
      component.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  public async deleteSupplier({ component, event }: any): Promise<void> {
    event.preventDefault();
    LoadingService.show();

    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const regionGrid = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');
    const parameters = component.selectedRows.map((row) => ({
      ...row,
      CDFILIAL: quoteGrid.datasource.currentRow.CDFILIAL,
      CDREGIAO: regionGrid.datasource.currentRow.CDREGIAO,
    }));

    try {
      await Http.delete('/deleteSupplier', { params: parameters });

      LoadingService.hide();

      this.utilities.showDialog('sup::SUCCESS', 'success');

      component.selectedRows = [];
      regionGrid.reload();
      component.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  public async updateProducts({ component, event }: any): Promise<void> {
    event.preventDefault();

    const editedRows = component.getEditedRows();

    if (!editedRows.length) {
      component.cancelEditedRows();
      return;
    }

    const { registeredParams } = this.appState;

    if (registeredParams.IDAUDALTCOT === 'S') {
      for (let i = 0; i < editedRows.length; i += 1) {
        if (
          editedRows[i].DSJUSTITM === null
            || editedRows[i].DSJUSTITM === undefined
        ) {
          this.utilities.showDialog('Justificativa não preenchida.', 'warning');

          return;
        }
      }
    }

    await this.showDialogs(registeredParams, editedRows);

    const params = this.prepareParamsUpdateProduct(editedRows);

    try {
      const { data } = await Http.post('/updateProducts', params);

      if (data.error) {
        this.utilities.showDialog(data.error, 'error');

        return;
      }

      component.cancelEditedRows();
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  private showDialogs(registeredParams, editedRows): Promise<void> {
    return new Promise((resolve) => {
      if (registeredParams.IDPERMREPLICREG === 'S') {
        this.showRegionDialog(registeredParams, editedRows).then(() => resolve());
      } else {
        editedRows.forEach((row) => {
          row.IDATUREG = false;
        });
        resolve();
      }
    });
  }

  private showRegionDialog(registeredParams, editedRows): Promise<void> {
    return new Promise((resolve) => {
      const confirmEvent: any = {
        click: () => {
          editedRows.forEach((row) => {
            row.IDATUREG = true;
          });

          if (registeredParams.IDPERMREPLICFOR === 'S') {
            this.showSupplierDialog(registeredParams, editedRows).then(() => resolve());
          } else {
            editedRows.forEach((row) => {
              row.IDATUFORN = false;
            });
            resolve();
          }
        },
      };

      const cancelEvent: any = {
        click: () => {
          editedRows.forEach((row) => {
            row.IDATUREG = false;
          });

          if (registeredParams.IDPERMREPLICFOR === 'S') {
            this.showSupplierDialog(registeredParams, editedRows).then(() => resolve());
          } else {
            editedRows.forEach((row) => {
              row.IDATUFORN = false;
            });
            resolve();
          }
        },
      };

      DialogService.show({
        name: 'buttons-dialog',
        type: 'warning',
        title: I18n.translate('Atenção'),
        text: I18n.translate('Deseja atualizar as alterações para as outras regiões?'),
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: I18n.translate('Sim'),
            events: confirmEvent,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: I18n.translate('Não'),
            events: cancelEvent,
          },
        ],
      });
    });
  }

  private showSupplierDialog(registeredParams, editedRows): Promise<void> {
    return new Promise((resolve) => {
      const confirmEvent: any = {
        click: () => {
          editedRows.forEach((row) => {
            row.IDATUFORN = true;
            DialogService.hide();
          });
          resolve();
        },
      };

      const cancelEvent: any = {
        click: () => {
          editedRows.forEach((row) => {
            row.IDATUFORN = false;
          });
          DialogService.hide();
          resolve();
        },
      };

      DialogService.show({
        name: 'buttons-dialog',
        type: 'warning',
        title: I18n.translate('Atenção'),
        text: I18n.translate('Deseja atualizar as alterações para as outros fornecedores?'),
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: I18n.translate('Sim'),
            events: confirmEvent,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: I18n.translate('Não'),
            events: cancelEvent,
          },
        ],
      });
    });
  }

  private prepareParamsUpdateProduct(editedRows) {
    const quoteSheetGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    editedRows.forEach((row) => {
      row.DTCOLEPREC = quoteSheetGrid.datasource.currentRow.DTCOLEPREC;
      row.CDFILIPROC = this.appState.CDFILCENTCMP;

      if (!row.DSJUSTITM) row.DSJUSTITM = '';
    });

    return editedRows;
  }

  public async deleteProduct({ component, event }: any): Promise<void> {
    event.preventDefault();
    LoadingService.show();

    try {
      await Http.delete('/deleteProduct', { params: component.selectedRows });
      LoadingService.hide();

      this.utilities.showDialog('sup::SUCCESS', 'success');

      component.selectedRows = [];
      component.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  public async excludeItemsWithZeroPrice(): Promise<void> {
    const supplierGrid = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');

    if (!supplierGrid.datasource.currentRow.CDFORNECED) {
      this.utilities.showDialog(
        'Selecione um fornecedor para excluir os itens com preços zerados.',
        'warning',
      );

      return;
    }

    LoadingService.show();

    try {
      const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
      const parameters = {
        CDFILIAL: quoteGrid.datasource.currentRow.CDFILIAL,
        NRCOLEPRECO: quoteGrid.datasource.currentRow.NRCOLEPRECO,
        NRORG: this.appState.NRORG,
      };

      const { data } = await Http.delete('/excludeItemsWithZeroPrice', { params: parameters });

      if (data.error) {
        LoadingService.hide();
        this.utilities.showDialog(data.error, 'error');

        return;
      }

      LoadingService.hide();
      this.utilities.showDialog('sup::SUCCESS', 'success');
      quoteGrid.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }

  public async loadPurchaseQuantity(): Promise<void> {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!quoteGrid.selectedRows.length) {
      this.utilities.showDialog('Selecione uma cotação para atualizar a quantidade a comprar.', 'warning');

      return;
    }

    LoadingService.show();

    try {
      const { data } = await Http.put('/loadPurchaseQuantity', { params: quoteGrid.selectedRows });

      if (data.error) {
        LoadingService.hide();
        this.utilities.showDialog(data.error, 'error');

        return;
      }

      LoadingService.hide();
      this.utilities.showDialog('sup::SUCCESS', 'success');
    } catch (error) {
      LoadingService.hide();
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }
}
