import {
  FormatterParserProvider, Http, I18n, dayjs,
} from '@zeedhi/core';
import { DialogService, LoadingService, FileInput } from '@zeedhi/common';
import { IReports, DialogTypeKeys, DialogType } from '@/util/Types';

export class Utilities {
  public formatValue(type: string, value: any): any {
    let valueReturn = '';

    if (value !== undefined) {
      const formatterFn = FormatterParserProvider.getFormatter('ZdNumber');

      switch (type) {
        case 'float':
          if (
            (value || value === 0)
            && value < 10000000000000
            && value > -10000000000000
          ) {
            valueReturn = formatterFn(value);
          } else {
            valueReturn = '-';
          }
          break;
        case 'number':
          valueReturn = value || value === 0 ? value.toFixed(0) : '';
          break;
        case 'date':
          valueReturn = value.substring(0, 10);
          break;
        default:
          valueReturn = value;
      }
    } else {
      valueReturn = value;
    }
    return valueReturn;
  }

  public formatFilter(obj: any): any {
    const objReturn = Object.entries(obj).map(([name, value]) => ({
      name,
      operator: '=',
      value,
    }));

    return objReturn;
  }

  public lightenColor(color: string, percent: number): string {
    const hex = color.replace('#', '');

    let r = parseInt(hex.substring(0, 2), 16);
    let g = parseInt(hex.substring(2, 4), 16);
    let b = parseInt(hex.substring(4, 6), 16);

    r = Math.floor((255 - r) * (percent / 100) + r);
    g = Math.floor((255 - g) * (percent / 100) + g);
    b = Math.floor((255 - b) * (percent / 100) + b);

    const lightenHex = '#'.concat(
      (r * 0x10000 + g * 0x100 + b).toString(16).padStart(6, '0'),
    );

    return lightenHex;
  }

  public async getLocalVar() {
    try {
      const response = await Http.get('/getLocalVar');

      return response.data.dataset.data;
    } catch (error) {
      return error;
    }
  }

  public validateProduct(initProduct, endProduct): { isValid: boolean, message?: string } {
    if (initProduct > endProduct) {
      return { isValid: false, message: 'Produto inicial deve ser menor ou igual ao produto final.' };
    }

    return { isValid: true };
  }

  public addCaracterInRight(code: string): string {
    const pad = 'ZZZZZZZZZZZZZZZZZZZZ';

    return code + pad.substr(0, pad.length - code.length);
  }

  public get productMask(): string {
    return '#.#.#.##.###.##';
  }

  public getSupplierMask(value: string): string {
    if (value.length <= 11) return '###.###.###-##';

    return '##.###.###/####-##';
  }

  public getWeekNumber(myDate: string): number {
    const date = dayjs(myDate);
    const januaryFirst = dayjs(new Date(date.year(), 0, 1));
    const numberOfDays = date.diff(januaryFirst, 'day');
    const result = Math.ceil((date.day() + 1 + numberOfDays) / 7);

    return result;
  }

  public showDialog(message: string, dialogTypeKey: DialogTypeKeys, isPersistent: boolean = true): void {
    DialogService.show({
      name: `${dialogTypeKey}-dialog`,
      type: dialogTypeKey,
      title: I18n.translate(DialogType[dialogTypeKey]),
      text: I18n.translate(message),
      persistent: isPersistent,
    });
  }

  public async deleteSelectedRows(
    component: any,
    selectedRows: any,
    route: string,
  ): Promise<void> {
    try {
      await Http.delete(`${route}/delete`, { data: selectedRows });
      component.selectedRows = [];
      component.reload();
    } catch (error) {
      this.showDialog(`Erro ao deletar cotação: ${error}`, 'error');
    }
  }

  public async updateRow(
    component: any,
    row: any,
    route: string,
  ): Promise<void> {
    try {
      await Http.post(route, row);
      component.cancelEditedRows();
      component.reload();
    } catch (error) {
      this.showDialog(`Erro ao atualizar cotação: ${error}`, 'error');
    }
  }

  public async convertXlsxToBase64(file: FileInput) {
    return new Promise((resolve, reject) => {
      const fileReader = new FileReader();

      fileReader.readAsDataURL(file.value[0]);

      fileReader.onload = () => resolve(fileReader.result);
      fileReader.onerror = reject;
    });
  }

  public getValidityDate({ component }: any): void {
    component.value = [dayjs(), dayjs()];
  }

  public async getReports(reportCode: string): Promise<IReports[]> {
    const { data } = await Http.post('/lib_getDataReportField', {
      requestType: 'Row',
      row: [reportCode],
    });

    return data.dataset.dataReport;
  }

  public async openReport(parameters): Promise<void> {
    LoadingService.show();

    try {
      const { data } = await Http.post('/generateReport', {
        requestType: 'Row',
        row: parameters,
      });
      const url = data.messages[0].message;

      window.open(url);
    } catch (error) {
      this.showDialog('sup::ERROR', 'error');
    } finally {
      LoadingService.hide();
    }
  }

  public async downloadReportXLSX(fileName: string, parameters): Promise<void> {
    LoadingService.show();

    try {
      const { data } = await Http.post('/generateReport', {
        requestType: 'Row',
        row: parameters,
      });
      const base64 = data.messages[0].message.split('&format=')[0];
      const link = `data:application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;base64,${base64}`;
      const anchor = document.createElement('a');

      anchor.href = link;
      anchor.download = fileName;

      document.body.appendChild(anchor);
      anchor.click();
      document.body.removeChild(anchor);
    } catch (error) {
      this.showDialog('sup::ERROR', 'error');
    } finally {
      LoadingService.hide();
    }
  }
}
